@@ -1,6 +1,5 @@
 import re
 import gc
-#import gwaslab as gl
 import pandas as pd
 import numpy as np
 from itertools import repeat
@@ -8,6 +7,8 @@ from multiprocessing import  Pool
 from liftover import get_lifter
 from functools import partial
 from gwaslab.vchangestatus import vchange_status
+from gwaslab.vchangestatus import status_match
+from gwaslab.vchangestatus import change_status
 from gwaslab.Log import Log
 from gwaslab.CommonData import get_chr_to_number
 from gwaslab.CommonData import get_number_to_chr
@@ -33,8 +34,8 @@ from gwaslab.CommonData import get_chr_list
 
 def fixID(sumstats,
        snpid="SNPID",rsid="rsID",chrom="CHR",pos="POS",nea="NEA",ea="EA",status="STATUS",
-       fixchrpos=False,fixid=False,fixeanea=False,fixeanea_flip=False,fixsep=False,overwrite=False,verbose=True,forcefixid=False,log=Log()):  
-    
+       fixchrpos=False,fixid=False,fixeanea=False,fixeanea_flip=False,fixsep=False,
+       overwrite=False,verbose=True,forcefixid=False,log=Log()):  
     '''
     1. fx SNPid
     2. fix chr and pos using snpid
@@ -49,7 +50,7 @@ def fixID(sumstats,
     if snpid in sumstats.columns:  
         if verbose: log.write(" -Checking if SNPID is chr:pos:ref:alt...(separator: - ,: , _)")
         #is_chrposrefalt = sumstats[snpid].str.match(r'(chr)?([0-9XYMT]+)[:_-]([0-9]+)[:_-]([ATCG]+)[:_-]([ATCG]+)', case=False, flags=0, na=False)
-        is_chrposrefalt = sumstats[snpid].str.match(r'\w+[:_-]\w+[:_-]\w+[:_-]\w+', case=False, flags=0, na=False)
+        is_chrposrefalt = sumstats[snpid].str.match(r'^\w+[:_-]\d+[:_-][ATCG]+[:_-][ATCG]+$', case=False, flags=0, na=False)
         is_snpid_na = sumstats[snpid].isna()
         sumstats.loc[ is_chrposrefalt,status] = vchange_status(sumstats.loc[ is_chrposrefalt,status],3 ,"975" ,"630")
         sumstats.loc[(~is_chrposrefalt)&(~is_snpid_na),status] = vchange_status(sumstats.loc[(~is_chrposrefalt)&(~is_snpid_na),status],3 ,"975" ,"842")
@@ -62,7 +63,7 @@ def fixID(sumstats,
         sumstats.loc[~is_rsid,status] = vchange_status(sumstats.loc[~is_rsid,status], 3, "986","743")
         
         if verbose: log.write(" -Checking if chr:pos:ref:alt is mixed in rsID column ...")
-        is_rs_chrpos = sumstats[rsid].str.match(r'\w+[:_-]\w+[:_-]\w+[:_-]\w+', case=False, flags=0, na=False)
+        is_rs_chrpos = sumstats[rsid].str.match(r'^\w+[:_-]\w+[:_-]\w+[:_-]\w+$', case=False, flags=0, na=False)
         #is_rs_chrpos = sumstats[rsid].str.match(r'(chr)?([0-9XYMT]+)[:_-]([0-9]+)[:_-]([ATCG]+)[:_-]([ATCG]+)', case=False, flags=0, na=False)
         
         if verbose: log.write(" -Number of chr:pos:ref:alt mixed in rsID column :",sum(is_rs_chrpos))
@@ -222,20 +223,27 @@ def fixID(sumstats,
             if overwrite is False:
                 #fix empty 
                 to_fix = sumstats[snpid].isna() & sumstats[status].str.match( r"\w\w\w[0]\w\w\w", case=False, flags=0, na=False ) 
+                #status_match(sumstats[status],4,0)
+                #
             else:
                 #fix all
                 to_fix = sumstats[status].str.match( r"\w\w\w[0]\w\w\w", case=False, flags=0, na=False ) 
+                # status_match(sumstats[status],4,0)
+                #
             
             if (ea in sumstats.columns) and (nea in sumstats.columns):
             # when ea and nea is available  -> check status -> fix to chr:pos:nea:ea 
                 
                 pattern = r"\w\w\w[0]\w\w\w"  
-                matched_index = sumstats[status].str.match(pattern) 
+                matched_index = sumstats[status].str.match(pattern)
+                #matched_index = status_match(sumstats[status],4,0) #
                 to_part_fix = matched_index & to_fix 
                 
                 #pattern = r"\w\w\w[0][01267][01234]\w"  
                 pattern = r"\w\w\w\w[0123][01267][01234]"
                 matched_index = sumstats[status].str.match(pattern)
+                #status_match(sumstats[status],5,[0,1,2,3]) | status_match(sumstats[status],6,[0,1,2,6,7])| status_match(sumstats[status],7,[0,1,2,3,4])
+                #
                 if forcefixid is True:
                     matched_index = to_fix
                 to_full_fix = matched_index & to_fix 
@@ -337,61 +345,52 @@ def removedup(sumstats,mode="dm",chrom="CHR",pos="POS",snpid="SNPID",ea="EA",nea
 ###############################################################################################################
 # 20220514
 def fixchr(sumstats,chrom="CHR",status="STATUS",add_prefix="",x="X",y="Y",mt="MT",remove=False, verbose=True,log=Log()):
+        chrom_list = get_chr_list() #bottom 
+
         if check_col(sumstats,chrom,status) is not True:
             if verbose: log.write(".fix_chr: Specified not detected..skipping...")
             return sumstats
         if verbose: log.write("Start to fix chromosome notation...")
         if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))          
-        # convert to string datatype
-        sumstats.loc[:,chrom] = sumstats.loc[:,chrom].astype("string") 
-        
-        is_chr_fixed = sumstats[chrom].str.match(r'[12]?[0-9]', case=False, flags=0, na=False)
-        
+        # convert to string datatype        
+        if sumstats.loc[:,chrom].dtype != "string":
+            sumstats.loc[:,chrom] = sumstats.loc[:,chrom].astype("string") 
+        #is_chr_fixed = sumstats[chrom].str.match(r'^[12]?[0-9]$', case=False, flags=0, na=False)
+        is_chr_fixed = sumstats[chrom].str.isnumeric()
+        if verbose: log.write(" -Vairants with standardized chromosome notation:",sum(is_chr_fixed))  
         if sum(is_chr_fixed)<len(sumstats):
-            is_chr_fixable = sumstats.loc[~is_chr_fixed,chrom].str.match(r'(chr)?([012][0-9]|[0-9]|X|Y|M|MT)', case=False, flags=0, na=False)
-            
+            #is_chr_fixable = sumstats.loc[~is_chr_fixed,chrom].str.match(r'^(chr)?([012][0-9]|[0-9]|X|Y|M|MT)$', case=False, flags=0, na=False)
+            chr_extracted = sumstats.loc[~is_chr_fixed,chrom].str.extract(r'(chr)?([012]?[0-9]|[XYM]|[MT])$',flags=re.IGNORECASE|re.ASCII)[1]
+            is_chr_fixable = ~chr_extracted.isna()
             if verbose: log.write(" -Vairants with fixable chromosome notations:",sum(is_chr_fixable))   
-            is_chr_na    = sumstats.loc[~is_chr_fixed,chrom].isna()
+            is_chr_na  = sumstats.loc[~is_chr_fixed,chrom].isna()
             if sum(is_chr_na)>0 and verbose: 
                 log.write(" -Vairants with NA chromosome notations:",sum(is_chr_na))  
             #not na and not fixable
             is_chr_invalid = (~is_chr_fixable)&(~is_chr_na)
             if sum(is_chr_invalid)>0 and verbose: 
                 log.write(" -Vairants with invalid chromosome notations:",sum(is_chr_invalid)) 
-
-            if sum(is_chr_fixable)>0:
-                # x,y,mt to X,Y,MT
-                if verbose: log.write(" -Converting to string datatype and UPPERCASE...") 
-                sumstats.loc[is_chr_fixable.index,chrom] =sumstats.loc[is_chr_fixable.index,chrom].str.upper()
-
-                # strip prefix
-                if verbose: log.write(" -Stripping chr prefix if exists : CHR_-.0...") 
-                #if sumstats.loc[:,chrom].dtype == "category":
-                #    sumstats.loc[:,chrom] = sumstats.loc[:,chrom].str.lstrip("CHR_-.0")
-                #else:
-                sumstats.loc[is_chr_fixable.index.values,chrom] = sumstats.loc[is_chr_fixable.index.values,chrom].str.lstrip("CHR_-.0")
-
-
-            # X, Y, MT to 23,24,25
-            sex_chr = sumstats[chrom].isin([x,y,mt])
-            if sum(sex_chr)>0:
-                if verbose: log.write(" -Identified ",str(sum(sex_chr))," variants on sex chromosomes...")
-                convert_num_to_xymt={x:"23",y:"24",mt:"25"}
-                sumstats.loc[sex_chr,chrom] =sumstats.loc[sex_chr,chrom].map(convert_num_to_xymt)
-                if verbose: log.write(" -Standardizing sex chromosome notations:" ,str(x),str(y),str(mt)," to 23,24,25...") 
-
-            chrom_list = get_chr_list() #bottom 
-            unrecognized_num = sum(~sumstats[chrom].isin(chrom_list))
-        
-            sumstats.loc[is_chr_fixed,status] = vchange_status(sumstats.loc[is_chr_fixed,status],4,"986","520")
-            sumstats.loc[is_chr_fixable.index,status] = vchange_status(sumstats.loc[is_chr_fixable.index,status],4,"986","520")
-            sumstats.loc[is_chr_invalid.index,status] = vchange_status(sumstats.loc[is_chr_invalid.index,status],4,"986","743")
-
-            if unrecognized_num>0 and verbose is True: 
-                log.write(" -Unrecognized chromosome notations :" , set(sumstats.loc[~sumstats[chrom].isin(chrom_list),chrom].head()) )
+                log.write(" -Invalid chromosome notations converted to NA :" , set(sumstats.loc[~sumstats[chrom].isin(chrom_list),chrom].head()))
             elif verbose:
                 log.write(" -No unrecognized chromosome notations...")
+            
+            sumstats.loc[is_chr_fixable.index,chrom] = chr_extracted[is_chr_fixable.index]
 
+            # X, Y, MT to 23,24,25
+            sex_chr = sumstats[chrom].isin([x.lower(),y.lower(),mt.lower(),x.upper(),y.upper(),mt.upper()])
+            if sum(sex_chr)>0:
+                if verbose: log.write(" -Identified ",str(sum(sex_chr))," variants on sex chromosomes...")
+                convert_num_to_xymt={x.lower():"23",y.lower():"24",mt.lower():"25",x.upper():"23",y.upper():"24",mt.upper():"25"}
+                sumstats.loc[sex_chr,chrom] =sumstats.loc[sex_chr,chrom].map(convert_num_to_xymt)
+                if verbose: log.write(" -Standardizing sex chromosome notations:" ,str(x),str(y),str(mt)," to 23,24,25...") 
+        
+            sumstats.loc[is_chr_fixed,status] = vchange_status(sumstats.loc[is_chr_fixed,status],4,"986","520")
+            if len(is_chr_fixable.index)>0:
+                sumstats.loc[is_chr_fixable.index,status] = vchange_status(sumstats.loc[is_chr_fixable.index,status],4,"986","520")
+            if len(is_chr_fixable.index)>0:
+                sumstats.loc[is_chr_invalid.index,status] = vchange_status(sumstats.loc[is_chr_invalid.index,status],4,"986","743")
+
+            unrecognized_num = sum(~sumstats[chrom].isin(chrom_list))
             if (remove is True) and unrecognized_num>0:
                 # remove variants with unrecognized chrom 
                 if verbose: log.write(" -Removed "+ str(unrecognized_num)+ " variants with unrecognized chromosome notations.") 
@@ -401,17 +400,12 @@ def fixchr(sumstats,chrom="CHR",status="STATUS",add_prefix="",x="X",y="Y",mt="MT
         else:
             if verbose: log.write(" -All CHR are already fixed...") 
             sumstats.loc[is_chr_fixed,status] = vchange_status(sumstats.loc[is_chr_fixed,status],4,"986","520")
-            
-        #if add_prefix:
-        #    if verbose: log.write(" -Adding prefix : "+ add_prefix+"...")
-        #    sumstats.loc[:,chrom] = add_prefix + sumstats[chrom]   
-
-        sumstats.loc[:,chrom] = np.floor(pd.to_numeric(sumstats.loc[:,chrom], errors='coerce')).astype('Int64')
-        #categories = [str(i) for i in range(26)]+[pd.NA]
-        #sumstats.loc[:,chrom]= pd.Categorical(sumstats.loc[:,chrom],categories=categories,ordered=True)
+        try:
+            sumstats.loc[:,chrom] = sumstats.loc[:,chrom].astype('Int64')
+        except:
+            sumstats.loc[:,chrom] = np.floor(pd.to_numeric(sumstats.loc[:,chrom], errors='coerce')).astype('Int64')
         if verbose: log.write("Finished fixing chromosome notation successfully!")
         return sumstats
-    
 
 ###############################################################################################################    
 # 20220514
@@ -426,22 +420,29 @@ def fixpos(sumstats,pos="POS",status="STATUS",remove=False, verbose=True,limit=2
         #convert to numeric
         is_pos_na = sumstats.loc[:,pos].isna()
         
-        sumstats.loc[:,pos] = np.floor(pd.to_numeric(sumstats.loc[:,pos], errors='coerce')).astype('Int64')
+        if sumstats[pos].dtype == "string" or sumstats[pos].dtype == "object":
+            if verbose: log.write(' -Removing thousands separator "," or underbar "_" ...')
+            sumstats.loc[~is_pos_na, pos] = sumstats.loc[~is_pos_na, pos].str.replace(",|_", "")
+
+        try:
+            if verbose: log.write(' -Converting to Int64 data type ...')
+            sumstats.loc[:,pos] = sumstats.loc[:,pos].astype('Int64')
+        except:
+            if verbose: log.write(' -Force converting to Int64 data type ...')
+            sumstats.loc[:,pos] = np.floor(pd.to_numeric(sumstats.loc[:,pos], errors='coerce')).astype('Int64')
         
         is_pos_fixed = ~sumstats.loc[:,pos].isna()
         is_pos_invalid = (~is_pos_na)&(~is_pos_fixed)
         
-        sumstats.loc[is_pos_fixed,status] = vchange_status(sumstats.loc[is_pos_fixed,status],4,"975","630")
+        sumstats.loc[is_pos_fixed,status]   = vchange_status(sumstats.loc[is_pos_fixed,status]  ,4,"975","630")
         sumstats.loc[is_pos_invalid,status] = vchange_status(sumstats.loc[is_pos_invalid,status],4,"975","842")
         
-        
         # remove outlier, limit:250,000,000
         if verbose: log.write(" -Position upper_bound is: " + "{:,}".format(limit))
         out_lier=(sumstats[pos]>limit) & (~is_pos_na)
         if verbose: log.write(" -Remove outliers:",sum(out_lier))
         sumstats = sumstats.loc[~out_lier,:]
         
-        
         #remove na
         if remove is True: 
             sumstats = sumstats.loc[~sumstats[pos].isna(),:]
@@ -472,7 +473,8 @@ def fixallele(sumstats,ea="EA", nea="NEA",status="STATUS",remove=False,verbose=T
         good_ea  = ~bad_ea
         good_nea = ~bad_nea
         is_eanea_na = sumstats[ea].isna() |  sumstats[nea].isna()
-        is_invalid = bad_ea | bad_nea | (sumstats[nea] == sumstats[ea])
+        not_variant = sumstats[nea] == sumstats[ea]
+        is_invalid = bad_ea | bad_nea | not_variant
         exclude  = bad_nea | bad_ea
         if verbose: 
             if len(set(sumstats.loc[bad_ea,ea].head())) >0:
@@ -496,13 +498,18 @@ def fixallele(sumstats,ea="EA", nea="NEA",status="STATUS",remove=False,verbose=T
         is_not_normalized = (sumstats[ea].str.len()>1) &(sumstats[nea].str.len()>1)
         is_normalized = is_indel &( (sumstats[ea].str.len()==1) &(sumstats[nea].str.len()>1) | (sumstats[ea].str.len()>1) &(sumstats[nea].str.len()==1) )
         
-        sumstats.loc[is_invalid, status]                 = vchange_status(sumstats.loc[is_invalid,status],                5,"9","6") 
-        sumstats.loc[is_eanea_na,status]                 = vchange_status(sumstats.loc[is_eanea_na, status],              5,"9","7")
-        sumstats.loc[is_eanea_fixed&is_not_normalized,status]   = vchange_status(sumstats.loc[is_eanea_fixed&is_not_normalized,status], 5,"9","5")
-        sumstats.loc[is_eanea_fixed&is_snp, status]          = vchange_status(sumstats.loc[is_eanea_fixed&is_snp,status],        5,"9","0")
-        sumstats.loc[is_eanea_fixed&is_indel,status]         = vchange_status(sumstats.loc[is_eanea_fixed&is_indel, status],      5,"9","4")
-        sumstats.loc[is_eanea_fixed&is_normalized,status]      = vchange_status(sumstats.loc[is_eanea_fixed&is_normalized, status],  5,"4","3")
-        remain_var_num = len(sumstats)
+        if sum(is_invalid)>0:
+            sumstats.loc[is_invalid, status]                      = vchange_status(sumstats.loc[is_invalid,status],                5,"9","6") 
+        if sum(is_eanea_na)>0:
+            sumstats.loc[is_eanea_na,status]                      = vchange_status(sumstats.loc[is_eanea_na, status],              5,"9","7")
+        if sum(is_eanea_fixed&is_not_normalized)>0:
+            sumstats.loc[is_eanea_fixed&is_not_normalized,status] = vchange_status(sumstats.loc[is_eanea_fixed&is_not_normalized,status], 5,"9","5")
+        if sum(is_eanea_fixed&is_snp)>0:
+            sumstats.loc[is_eanea_fixed&is_snp, status]           = vchange_status(sumstats.loc[is_eanea_fixed&is_snp,status],        5,"9","0")
+        if sum(is_eanea_fixed&is_indel)>0:
+            sumstats.loc[is_eanea_fixed&is_indel,status]          = vchange_status(sumstats.loc[is_eanea_fixed&is_indel, status],      5,"9","4")
+        if sum(is_eanea_fixed&is_normalized)>0:
+            sumstats.loc[is_eanea_fixed&is_normalized,status]     = vchange_status(sumstats.loc[is_eanea_fixed&is_normalized, status],  5,"4","3")
         gc.collect()
         if verbose: log.write("Finished fixing allele successfully!")
         return sumstats
@@ -510,14 +517,16 @@ def fixallele(sumstats,ea="EA", nea="NEA",status="STATUS",remove=False,verbose=T
 ###############################################################################################################   
 # 20220721
 
-def parallelnormalizeallele(sumstats,pos="POS",nea="NEA",ea="EA" ,status="STATUS",n_cores=1,verbose=True,log=Log()):
+def parallelnormalizeallele(sumstats,snpid="SNPID",rsid="rsID",pos="POS",nea="NEA",ea="EA" ,status="STATUS",n_cores=1,verbose=True,log=Log()):
     if check_col(sumstats,pos,ea,nea,status) is not True:
         if verbose: log.write(".normalize(): specified columns not detected..skipping...")
         return sumstats
     
     if verbose: log.write("Start to normalize variants...")
     if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
-    variants_to_check = sumstats[status].str.match(r'\w\w\w\w[45]\w\w', case=False, flags=0, na=False)
+    #variants_to_check = status_match(sumstats[status],5,[4,5]) #
+    #r'\w\w\w\w[45]\w\w'
+    variants_to_check = sumstats[status].str[4].str.match(r'4|5', case=False, flags=0, na=False)
     if sum(variants_to_check)==0:
         if verbose: log.write(" -No available variants to normalize..")
         if verbose: log.write("Finished normalizing variants successfully!")
@@ -538,8 +547,17 @@ def parallelnormalizeallele(sumstats,pos="POS",nea="NEA",ea="EA" ,status="STATUS
         before_normalize = sumstats.loc[variants_to_check,[ea,nea]]
         changed_num = len(normalized_pd.loc[(before_normalize[ea]!=normalized_pd[ea]) | (before_normalize[nea]!=normalized_pd[nea]),:])
         if changed_num>0:
-            log.write(" -Not normalized allele:",end="")
+            if snpid in sumstats.columns:
+                before_normalize_id = sumstats.loc[variants_to_check,snpid]
+            elif rsid in sumstats.columns:
+                before_normalize_id = sumstats.loc[variants_to_check,rsid]
             
+            log.write(" -Not normalized allele IDs:",end="")
+            for i in before_normalize_id.loc[(before_normalize[ea]!=normalized_pd[ea]) | (before_normalize[nea]!=normalized_pd[nea])].head().values:
+                log.write(i,end=" ",show_time=False)
+            log.write("... \n",end="",show_time=False) 
+
+            log.write(" -Not normalized allele:",end="")
             for i in before_normalize.loc[(before_normalize[ea]!=normalized_pd[ea]) | (before_normalize[nea]!=normalized_pd[nea]),[ea,nea]].head().values:
                 log.write(i,end="",show_time=False)
             log.write("... \n",end="",show_time=False)     
@@ -548,16 +566,14 @@ def parallelnormalizeallele(sumstats,pos="POS",nea="NEA",ea="EA" ,status="STATUS
             log.write(" -All variants are already normalized..")
     ###################################################################################################################
     categories = set(sumstats.loc[:,ea])|set(sumstats.loc[:,nea]) |set(normalized_pd.loc[:,ea]) |set(normalized_pd.loc[:,nea])
-    sumstats.loc[:,ea]=pd.Categorical(sumstats.loc[:,ea],categories = categories) 
-    sumstats.loc[:,nea]=pd.Categorical(sumstats.loc[:,nea],categories = categories ) 
+    sumstats.loc[:,ea]  = pd.Categorical(sumstats.loc[:,ea],categories = categories) 
+    sumstats.loc[:,nea] = pd.Categorical(sumstats.loc[:,nea],categories = categories ) 
     sumstats.loc[variants_to_check,[pos,nea,ea,status]] = normalized_pd.values
-    sumstats.loc[:,pos] = np.floor(pd.to_numeric(sumstats.loc[:,pos], errors='coerce')).astype('Int64')
-    #sumstats.loc[:,[nea,ea,status]] = sumstats.loc[:,[nea,ea,status]].astype("string")
-    #if verbose:log.write(" -Updating allele status...")
-    #sumstats.loc[:,status] = vchange_status(sumstats.loc[:,status], 5,"5","3")
-    #indel_to_snp = sumstats[status].str.match(r'\w\w\w\w[3]\w\w', case=False, flags=0, na=False) & (sumstats[ea].str.len()==1) &(sumstats[nea].str.len()==1)
-    #if sum(indel_to_snp)>0:
-    #    sumstats.loc[indel_to_snp,status] = vchange_status(sumstats.loc[indel_to_snp,status], 5,"5","3")
+    try:
+        sumstats.loc[:,pos] = sumstats.loc[:,pos].astype('Int64')
+    except:
+        sumstats.loc[:,pos] = np.floor(pd.to_numeric(sumstats.loc[:,pos], errors='coerce')).astype('Int64')
+  
     if verbose: log.write("Finished normalizing variants successfully!")
     return sumstats
 
@@ -578,8 +594,8 @@ def normalizevariant(pos,a,b,status):
     status_end=status[5:]
     
     if len(a)==1 or len(b)==1:
+        #return pos,a,b,change_status(status,5,0) #status_pre+"0"+status_end
         return pos,a,b,status_pre+"0"+status_end
-    
     pos_change=0
     pointer_a_l, pointer_a_r = 0, len(a)-1
     pointer_b_l, pointer_b_r = 0, len(b)-1
@@ -593,8 +609,8 @@ def normalizevariant(pos,a,b,status):
             break
         if pointer_a_r-pointer_a_l==0 or pointer_b_r-pointer_b_l==0:
             if len(a)==1 or len(b)==1:
-                return pos,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"0"+status_end
-            return pos,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"3"+status_end
+                return pos,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"0"+status_end #change_status(status,5,0) #
+            return pos,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"3"+status_end #change_status(status,5,3) #
 
     # remove from left
     for i in range(max(len(a),len(b))):
@@ -608,8 +624,8 @@ def normalizevariant(pos,a,b,status):
         if pointer_a_r-pointer_a_l==0 or pointer_b_r-pointer_b_l==0:
             break
     if len(a)==1 or len(b)==1:
-        return pos+pos_change,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"0"+status_end
-    return pos+pos_change,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"3"+status_end
+        return pos+pos_change,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"0"+status_end #change_status(status,5,0) #
+    return pos+pos_change,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"3"+status_end # change_status(status,5,3) #
 ""
 
 
@@ -766,11 +782,10 @@ def sanitycheckstats(sumstats,
     if "STATUS" in coltocheck and "STATUS" in sumstats.columns:
         cols_to_check.append("STATUS")
         if verbose: log.write(" -Checking STATUS...") 
+        #sumstats["STATUS"] = sumstats["STATUS"].astype("Int64")
         categories = {str(j+i) for j in [1900000,3800000,9700000,9800000,9900000] for i in range(0,100000)}
         sumstats.loc[:,"STATUS"] = pd.Categorical(sumstats["STATUS"],categories=categories)
-        if verbose: log.write(" -Coverting STAUTUS to category.") 
-    
-    
+        if verbose: log.write(" -Coverting STAUTUS to interger.") 
     
     sumstats = sumstats.dropna(subset=cols_to_check)
     after_number=len(sumstats)
@@ -807,7 +822,8 @@ def flipallelestats(sumstats,status="STATUS",verbose=True,log=Log()):
     
     ###################get reverse complementary####################
     pattern = r"\w\w\w\w\w[45]\w"  
-    matched_index = sumstats[status].str.match(pattern)
+    #matched_index = status_match(sumstats[status],6,[4,5]) #
+    matched_index = sumstats[status].str[5].str.match(r"4|5")
     if sum(matched_index)>0:
         if verbose: log.write("Start to convert alleles to reverse complement for SNPs with status xxxxx[45]x... ") 
         if verbose: log.write(" -Flipping "+ str(sum(matched_index)) +" variants...") 
@@ -825,7 +841,8 @@ def flipallelestats(sumstats,status="STATUS",verbose=True,log=Log()):
 
     ###################flip ref####################
     pattern = r"\w\w\w\w\w[35]\w"  
-    matched_index = sumstats[status].str.match(pattern)
+    #matched_index = status_match(sumstats[status],6,[3,5]) #sumstats[status].str.match(pattern)
+    matched_index = sumstats[status].str[5].str.match(r"3|5")
     if sum(matched_index)>0:
         if verbose: log.write("Start to flip allele-specific stats for SNPs with status xxxxx[35]x: alt->ea , ref->nea ... ") 
         if verbose: log.write(" -Flipping "+ str(sum(matched_index)) +" variants...") 
@@ -856,7 +873,8 @@ def flipallelestats(sumstats,status="STATUS",verbose=True,log=Log()):
         
     ###################flip ref for undistingushable indels####################
     pattern = r"\w\w\w\w[123][67]6"  
-    matched_index = sumstats[status].str.match(pattern)
+    #matched_index = status_match(sumstats[status],6,[1,2,3])|status_match(sumstats[status],6,[6,7])|status_match(sumstats[status],7,6) #sumstats[status].str.match(pattern)
+    matched_index = sumstats[status].str[4:].str.match(r"[123][67]6")
     if sum(matched_index)>0:
         if verbose: log.write("Start to flip allele-specific stats for standardized indels with status xxxx[123][67][6]: alt->ea , ref->nea ... ") 
         if verbose: log.write(" -Flipping "+ str(sum(matched_index)) +" variants...") 
@@ -887,7 +905,8 @@ def flipallelestats(sumstats,status="STATUS",verbose=True,log=Log()):
          # flip ref
     ###################flip statistics for reverse strand panlindromic variants####################
     pattern = r"\w\w\w\w\w[012]5"  
-    matched_index = sumstats[status].str.match(pattern)
+    #matched_index = status_match(sumstats[status],6,[0,1,2]) | status_match(sumstats[status],7,[5])#sumstats[status].str.match(pattern)
+    matched_index = sumstats[status].str[5:].str.match(r"05|15|25")
     if sum(matched_index)>0:
         if verbose: log.write("Start to flip allele-specific stats for palindromic SNPs with status xxxxx[12]5: (-)strand <=> (+)strand ... ") 
         if verbose: log.write(" -Flipping "+ str(sum(matched_index)) +" variants...") 
@@ -939,7 +958,6 @@ def liftover_variant(sumstats,
              status="STATUS",
              from_build="19", 
              to_build="38"):
-    
     converter = get_lifter("hg"+from_build,"hg"+to_build)
     dic= get_number_to_chr(in_chr=False,xymt=["X","Y","M"])
     dic2= get_chr_to_number(out_chr=False)
@@ -947,12 +965,9 @@ def liftover_variant(sumstats,
         chrom_to_convert = dic[i]
         variants_on_chrom_to_convert = sumstats[chrom]==i
         lifted = sumstats.loc[variants_on_chrom_to_convert,[pos,status]].apply(lambda x: liftover_snv(x[[pos,status]],chrom_to_convert,converter,to_build),axis=1)
-        sumstats.loc[sumstats[chrom]==i,pos]     =  lifted.str[1]
+        sumstats.loc[sumstats[chrom]==i,pos]     =   lifted.str[1]
         sumstats.loc[sumstats[chrom]==i,status]   =  lifted.str[2]
         sumstats.loc[sumstats[chrom]==i,chrom]    =  lifted.str[0].map(dic2)
-    #sumstats.loc[:,chrom]  = #lifted.apply(lambda x :pd.NA if x[0] is pd.NA else str(x[0])).astype("string")
-    #sumstats.loc[:,pos]    = #lifted.apply(lambda x :pd.NA if x[0] is pd.NA else str(x[1]))
-    #sumstats.loc[:,status] = #lifted.apply(lambda x :str(x[2])).astype("string")  
     return sumstats
 
 def parallelizeliftovervariant(sumstats,n_cores=1,chrom="CHR", pos="POS", from_build="19", to_build="38",status="STATUS",remove=True, verbose=True,log=Log()):
@@ -1003,46 +1018,22 @@ def sortcoordinate(sumstats,chrom="CHR",pos="POS",reindex=True,verbose=True,log=
         if verbose: log.write(".liftover(): specified columns not detected..skipping...")
         return sumstats
     
-    #chromosome_number_to_string = {i:str(i) for i in range(1,23)}
-    #chromosome_number_to_string[23] = "X"
-    #chromosome_number_to_string[24] = "Y"
-    #chromosome_number_to_string[25] = "MT"
-    
-    #chromosome_string_to_number = {str(i):i for i in range(1,23)}
-    #chromosome_string_to_number["X"] = 23
-    #chromosome_string_to_number["Y"] = 24
-    #chromosome_string_to_number["MT"] = 25
-    
-    #for i in sumstats[chrom]:
-    #    if i not in chromosome_string_to_number.keys():
-    #        chromosome_string_to_number[i]=i
-    #        chromosome_number_to_string[i]=i
-
-    #good_chrpos = sumstats[chrom].isin(get_chr_list())&sumstats[pos].notnull()
-    #sumstats_good = sumstats.loc[good_chrpos,:]
-    #sumstats_bad = sumstats.loc[~good_chrpos,:]
-    
-    #sumstats = sumstats.sort_values(by=[chrom,pos],ascending=True,ignore_index=True)
-    #####################????????????)
-    
-    
     if verbose: log.write("Start to sort the genome coordinates...")
     if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
     
-    #if verbose: log.write(" -Mapping chr1-22,X,Y,MT to numbers 1-25, other chromosome notations will be sorted as strings...")
-    
-    #sumstats[chrom] = sumstats[chrom].map(chromosome_string_to_number)
-    if verbose: log.write(" -Force converting POS to integers...")
-    #sumstats[chrom] = np.floor(pd.to_numeric(sumstats[chrom], errors='coerce')).astype('Int64')
-    sumstats[pos]  = np.floor(pd.to_numeric(sumstats[pos], errors='coerce')).astype('Int64')
+    try:
+        if sumstats[pos].dtype == "Int64":
+            pass
+        else:
+            if verbose: log.write(" -Force converting POS to Int64...")
+            sumstats[pos]  = np.floor(pd.to_numeric(sumstats[pos], errors='coerce')).astype('Int64')
+    except:
+        pass
+
     if verbose: log.write(" -Sorting genome coordinates...")
     sumstats = sumstats.sort_values(by=[chrom,pos],ascending=True,ignore_index=True)
-    
-    #if verbose: log.write(" -Converting chromosome column back to string type...")
-    
-    #sumstats[chrom] = sumstats[chrom].map(chromosome_number_to_string)
-    #sumstats[chrom] = sumstats[chrom].astype("string")
     if verbose: log.write("Finished sorting genome coordinates successfully!")
+    gc.collect()
     return sumstats
 ###############################################################################################################
 # 20221105
@@ -1063,7 +1054,6 @@ def sortcolumn(sumstats,verbose=True,log=Log(),order = [
     if verbose: log.write("Finished sorting columns successfully!")
     return sumstats
 
-
 def check_col(df,*args):
     not_in_df=[]
     for i in args:
@@ -1084,7 +1074,4 @@ def check_col(df,*args):
     if len(not_in_df)>0:
         return False
         print(" -Specified columns names was not detected. Please check:"+",".join(not_in_df))
-    return True
-            
-    
-    
+    return True
\ No newline at end of file
