@@ -279,7 +279,7 @@ def isfile_casesensitive(path):
     return filename in os.listdir(directory)
 
 def get_readargs_header(inpath,readargs):
-    if inpath.endswith("vcf.gz"):
+    if "vcf.gz" in inpath:
         with gzip.open(inpath,'r') as file:      
             skip=0
             for line in file:        
@@ -295,13 +295,16 @@ def get_readargs_header(inpath,readargs):
     return readargs_header
 
 def get_skip_rows(inpath):
-    with gzip.open(inpath,'r') as file:      
-        skip=0
-        for line in file:        
-            if line.decode('utf-8').startswith('##'):
-                skip+=1
-            else:
-                return skip
+    if "vcf.gz" in inpath:
+        with gzip.open(inpath,'r') as file:      
+            skip=0
+            for line in file:        
+                if line.decode('utf-8').startswith('##'):
+                    skip+=1
+                else:
+                    return skip
+    else:
+        return 0
 
 def parse_vcf_study(sumstats,format_cols,study,vcf_usecols,log,verbose=True):
     if verbose: log.write(" -Parsing based on FORMAT: ", format_cols)
@@ -354,15 +357,14 @@ def process_allele(sumstats,log,verbose):
             sumstats.loc[sumstats["EA"]==sumstats["ALT"],"NEA"] = sumstats.loc[sumstats["EA"]==sumstats["ALT"],"REF"]
             sumstats.loc[sumstats["EA"]!=sumstats["ALT"],"NEA"] = sumstats.loc[sumstats["EA"]!=sumstats["ALT"],"ALT"]
             sumstats = sumstats.drop(labels=["REF","ALT"],axis=1)
-        sumstats["EA"]=sumstats["EA"].fillna("N")
         sumstats["EA"]=sumstats["EA"].astype("category")     
     if "NEA" in sumstats.columns:
-        sumstats["NEA"]=sumstats["NEA"].fillna("N")
         sumstats["NEA"]=sumstats["NEA"].astype("category")  
     return sumstats
 
 def process_status(sumstats,build,log,verbose):
     if verbose: log.write(" -Initiating a status column: STATUS ...")
+    #sumstats["STATUS"] = int(build)*(10**5) +99999
     sumstats["STATUS"] = build +"99999"
     categories = {str(j+i) for j in [1900000,3800000,9700000,9800000,9900000] for i in range(0,100000)}
     sumstats["STATUS"] = pd.Categorical(sumstats["STATUS"],categories=categories)
